@@ -7,8 +7,8 @@
  * any later version.
  *
  * @author Lawrence Murray <lawrence@indii.org>
- * $Rev: 515 $
- * $Date: 2013-09-29 19:04:13 +0800 (Sun, 29 Sep 2013) $
+ * $Rev$
+ * $Date$
  */
 #include "Window.hpp"
 
@@ -55,16 +55,7 @@ Window::Window(wxWindow* parent, const Mode mode, WindowObserver* obs) :
 }
 
 Window::~Window() {
-  /* save AUI perspective */
-  wxConfig config(_(PACKAGE_TARNAME), _("indii.org"));
-  wxString persp = manager->SavePerspective();
-  config.Write(_("Perspective_v2"), persp);
-  config.Write(_("x"), GetPosition().x);
-  config.Write(_("y"), GetPosition().y);
-  config.Write(_("width"), GetSize().GetWidth());
-  config.Write(_("height"), GetSize().GetHeight());
-  
-  manager->UnInit();
+  //
 }
 
 void Window::openAndAdd(const wxString& file) {
@@ -153,6 +144,17 @@ void Window::OnSystemClose(wxCloseEvent& evt) {
     if (obs != NULL) {
       obs->notifyClose(this, exiting);
     }
+    
+    /* save AUI perspective */
+    wxConfig config(_(PACKAGE_TARNAME), _("indii.org"));
+    wxString persp = manager->SavePerspective();
+    config.Write(_("Perspective_v2"), persp);
+    config.Write(_("x"), GetPosition().x);
+    config.Write(_("y"), GetPosition().y);
+    config.Write(_("width"), GetSize().GetWidth());
+    config.Write(_("height"), GetSize().GetHeight());
+    manager->UnInit();
+    
     Destroy();
   } else {
     exiting = false;
@@ -513,6 +515,9 @@ void Window::init(Document* doc) {
   mnPopupWindow = new WindowMenu(this);
 
   /* toolbar */
+  #ifndef __WXGTK__
+  gnome_scale_images(this->GetContentScaleFactor());
+  #endif
   wxToolBar* tbTop = this->CreateToolBar(wxNO_BORDER|wxTB_FLAT|wxTB_HORIZONTAL);
   tbTop->SetToolBitmapSize(wxSize(32,32));
   if (mode == STANDALONE) {
@@ -529,7 +534,8 @@ void Window::init(Document* doc) {
   tbTop->AddTool(wxID_ZOOM_FIT, _("Zoom to Fit"), *_img_gnome_zoom_best_fit, _("Zoom to Fit"));
   tbTop->AddSeparator();
   #ifdef URBANLIGHTSCAPE
-  tbTop->AddCheckTool(ID_BUT_OVERLAY, _("Toggle Overlay"), *_img_gnome_window_new, *_img_gnome_window_new, _("Toggle Overlay"));
+  tbTop->AddCheckTool(ID_BUT_OVERLAY, _("Toggle Overlay"), *_img_gnome_window_new_32, *_img_gnome_window_new, _("Toggle Overlay"));
+  // ^ first image is *_32 as otherwise appears shrunk in one quarter on Mac OS X
   tbTop->ToggleTool(ID_BUT_OVERLAY, true);
   #endif
   if (mode == EMBED) {
