@@ -12,6 +12,8 @@
  */
 #include "Image.hpp"
 
+#include "ImageManipulation.hpp"
+
 #include "boost/numeric/ublas/matrix_proxy.hpp"
 
 indii::Image::Image(const int width, const int height) :
@@ -19,14 +21,21 @@ indii::Image::Image(const int width, const int height) :
   //
 }
 
-indii::Image::Image(const Image& img, const int scale) :
-    r(img.getHeight()/scale, img.getWidth()/scale),
-    g(img.getHeight()/scale, img.getWidth()/scale),
-    b(img.getHeight()/scale, img.getWidth()/scale) {
-  using namespace boost::numeric::ublas;
-
-  noalias(r) = subslice(img.r, 0, scale, r.size1(), 0, scale, r.size2());
-  noalias(g) = subslice(img.g, 0, scale, g.size1(), 0, scale, g.size2());
-  noalias(b) = subslice(img.b, 0, scale, b.size1(), 0, scale, b.size2());
+indii::Image::Image(const Image& img, const float scale) {
+  const int width = std::floor(img.getWidth()/scale);
+  const int height = std::floor(img.getHeight()/scale);
+     
+  r.resize(width, height, false);
+  g.resize(width, height, false);
+  b.resize(width, height, false);
+ 
+  for (int x = 0; x < width; ++x) {
+    for (int y = 0; y < height; ++y) {
+      int y1 = std::floor(y*scale);
+      int x1 = std::floor(x*scale);
+      r(y,x) = img.r(y1, x1);
+      g(y,x) = img.g(y1, x1);
+      b(y,x) = img.b(y1, x1);
+    }
+  }
 }
-
