@@ -35,7 +35,7 @@ public:
    * @param img Existing image.
    * @param scale Inverse scale of the new image.
    */
-  Image(const Image& img, const int scale = 1);
+  Image(const Image& img, const float scale = 1);
 
   /**
    * Width of image.
