@@ -7,8 +7,8 @@
  * any later version.
  * 
  * @author Lawrence Murray <lawrence@indii.org>
- * $Rev: 511 $
- * $Date: 2013-09-28 15:50:18 +0800 (Sat, 28 Sep 2013) $
+ * $Rev$
+ * $Date$
  */
 #include "ThumbImage.hpp"
 
@@ -23,12 +23,12 @@
 using namespace indii;
 
 ThumbImage::ThumbImage(wxWindow *parent, const unsigned int cluster) :
-    wxControl(parent, -1, wxDefaultPosition, wxDefaultSize, wxBORDER_NONE),
-    model(NULL), cluster(cluster), isActive(false), isOver(false),
-    fg(ICON_SIZE,ICON_SIZE), bg(ICON_SIZE,ICON_SIZE) {
+    wxControl(parent, -1, wxDefaultPosition, wxDefaultSize, wxBORDER_NONE), model(
+    NULL), cluster(cluster), isActive(false), isOver(false), fg(ICON_SIZE,
+    ICON_SIZE), bg(ICON_SIZE, ICON_SIZE) {
   SetBackgroundStyle(wxBG_STYLE_CUSTOM);
-  SetSize(ICON_SIZE,ICON_SIZE);
-  SetMinSize(wxSize(ICON_SIZE,ICON_SIZE));
+  SetSize(ICON_SIZE, ICON_SIZE);
+  SetMinSize(wxSize(ICON_SIZE, ICON_SIZE));
 
   fgPrepared = false;
 }
@@ -74,7 +74,7 @@ void ThumbImage::notifyClusterChange(const unsigned int i) {
 
 void ThumbImage::notifySatChange(const unsigned int i) {
   /* pre-condition */
-  assert (model != NULL);
+  assert(model != NULL);
 
   if (cluster == i && model->isShown(cluster) != isActive) {
     isActive = model->isShown(cluster);
@@ -84,7 +84,7 @@ void ThumbImage::notifySatChange(const unsigned int i) {
 
 void ThumbImage::OnPaint(wxPaintEvent& evt) {
   /* pre-condition */
-  assert (model != NULL);
+  assert(model != NULL);
 
   if (cluster < model->getNumClusters()) {
     if (!fgPrepared) {
@@ -95,37 +95,59 @@ void ThumbImage::OnPaint(wxPaintEvent& evt) {
     x = (ICON_SIZE - THUMB_SIZE) / 2;
     y = (ICON_SIZE - THUMB_SIZE) / 2;
 
+#ifndef __WXGTK__
+    double contentScale = this->GetContentScaleFactor();
+#else
+    double contentScale = 1.0;
+#endif
+    int thumbSize = THUMB_SIZE * contentScale;
+
     wxPaintDC dc(this);
-    #ifndef WX_28
+#ifndef WX_28
     dc.Clear();
-    #endif
+#endif
     if (isActive) {
       dc.SetBrush(wxBrush(*wxWHITE));
       dc.SetPen(wxPen(model->getColour(cluster), 3));
     } else if (isOver) {
       dc.SetBrush(wxBrush(*wxWHITE));
-      dc.SetPen(wxPen(wxColor(128,128,128), 2));      
+      dc.SetPen(wxPen(wxColor(128,128,128), 2));
     } else {
       dc.SetBrush(wxBrush(*wxWHITE));
       dc.SetPen(wxPen(wxColor(128,128,128), 1));
     }
-      
+
     dc.DrawRoundedRectangle(2, 2, ICON_SIZE - 4, ICON_SIZE - 4, 10);
-    
+
+    #ifdef __WXGTK__
     dc.DrawBitmap(bg, x, y);
     dc.DrawBitmap(fg, x, y);
+    #else
+    wxBitmap bmp1(bg);
+    wxMemoryDC mem1;
+    mem1.SelectObject(bmp1);
+
+    wxBitmap bmp2(fg);
+    wxMemoryDC mem2;
+    mem2.SelectObject(bmp2);
+
+    dc.StretchBlit(x, y, THUMB_SIZE, THUMB_SIZE, &mem1, 0, 0, thumbSize,
+        thumbSize);
+    dc.StretchBlit(x, y, THUMB_SIZE, THUMB_SIZE, &mem2, 0, 0, thumbSize,
+        thumbSize);
+    #endif
   }
 }
 
 void ThumbImage::OnLeftDown(wxMouseEvent& evt) {
   isActive = !isActive;
-  
+
   Refresh();
 }
 
 void ThumbImage::OnLeftUp(wxMouseEvent& evt) {
   /* pre-condition */
-  assert (model != NULL);
+  assert(model != NULL);
 
   isActive = !model->isShown(cluster);
   Refresh();
@@ -134,46 +156,60 @@ void ThumbImage::OnLeftUp(wxMouseEvent& evt) {
 }
 
 void ThumbImage::OnEnter(wxMouseEvent& evt) {
-  isOver = true;  
+  isOver = true;
   Refresh();
 }
 
 void ThumbImage::OnLeave(wxMouseEvent& evt) {
-  isOver = false;  
+  isOver = false;
   Refresh();
 }
 
 void ThumbImage::prepareBackground() {
   /* pre-condition */
-  assert (model != NULL);
+  assert(model != NULL);
 
   /* sizes, dimensions */
   ImageResource* res = model->getImageResource();
-  double ratio = static_cast<double>(res->getWidth())/res->getHeight();
+  double ratio = static_cast<double>(res->getWidth()) / res->getHeight();
+#ifndef __WXGTK__
+  double contentScale = this->GetContentScaleFactor();
+#else
+  double contentScale = 1.0;
+#endif
+  int thumbSize = THUMB_SIZE * contentScale;
 
   /* crop */
   wxImage* img;
   wxRect crop;
 
   if (ratio >= 1.0) {
-    img = res->getDisplay(intround(THUMB_SIZE*ratio));
-    crop = wxRect((img->GetWidth() - img->GetHeight()) / 2, 0, THUMB_SIZE, THUMB_SIZE);
+    img = res->getDisplay(intround(thumbSize * ratio));
+    crop = wxRect((img->GetWidth() - img->GetHeight()) / 2, 0, thumbSize,
+        thumbSize);
   } else {
-    img = res->getDisplay(THUMB_SIZE);
-    crop = wxRect(0, (img->GetHeight() - img->GetWidth()) / 2, THUMB_SIZE, THUMB_SIZE);
+    img = res->getDisplay(thumbSize);
+    crop = wxRect(0, (img->GetHeight() - img->GetWidth()) / 2, thumbSize,
+        thumbSize);
   }
   bg = wxBitmap(img->GetSubImage(crop).ConvertToGreyscale());
 }
 
 void ThumbImage::prepareForeground() {
   /* pre-condition */
-  assert (model != NULL);
+  assert(model != NULL);
 
   /* sizes, dimensions */
   ImageResource* res = model->getImageResource();
-  int scale = std::max(res->fitScale(THUMB_SIZE, THUMB_SIZE) - 1, 1);
-  int thumbWidth = res->getWidth()/scale;
-  int thumbHeight = res->getHeight()/scale;
+#ifndef __WXGTK__
+  double contentScale = this->GetContentScaleFactor();
+#else
+  double contentScale = 1.0;
+#endif
+  int thumbSize = THUMB_SIZE * contentScale;
+  int scale = std::max(res->fitScale(thumbSize, thumbSize) - 1, 1);
+  int thumbWidth = res->getWidth() / scale;
+  int thumbHeight = res->getHeight() / scale;
 
   /* rectangle for crop */
   wxRect crop;
@@ -194,40 +230,39 @@ void ThumbImage::prepareForeground() {
 
   wxMemoryDC dc1(fg1);
   wxMemoryDC dc2(fg2);
-  
+
   wxColour c1(model->getColour(cluster));
   wxColour c2;
   wxImage::RGBValue rgb(c1.Red(), c1.Green(), c1.Blue());
   wxImage::HSVValue hsv;
-  
+
   hsv = wxImage::RGBtoHSV(rgb);
   hsv.value *= 0.9;
   rgb = wxImage::HSVtoRGB(hsv);
   c2.Set(rgb.red, rgb.green, rgb.blue);
-  
+
   dc1.SetBackground(wxBrush(c1));
   dc2.SetBackground(wxBrush(c2));
-  dc1.Clear();  
+  dc1.Clear();
   dc2.Clear();
 
   wxImage img1(fg1.ConvertToImage());
   sparse_mask mask(model->calcMask(cluster, crop, scale));
   hide(img1);
   show(img1, mask);
-  
+
   dc2.DrawBitmap(img1, 0, 0, true);
-  
+
   wxImage img2(fg2.ConvertToImage());
   a.resize(crop.height, crop.width);
   model->calcAlpha(cluster, crop, scale, a);
   applyAlpha(img2, a);
-  fg = wxBitmap(img2.Scale(THUMB_SIZE, THUMB_SIZE, wxIMAGE_QUALITY_HIGH));
-  
+  fg = wxBitmap(img2.Scale(thumbSize, thumbSize, wxIMAGE_QUALITY_HIGH));
+
   fgPrepared = true;
 }
 
-BEGIN_EVENT_TABLE(ThumbImage, wxWindow)
-EVT_PAINT(ThumbImage::OnPaint)
+BEGIN_EVENT_TABLE(ThumbImage, wxWindow) EVT_PAINT(ThumbImage::OnPaint)
 //EVT_LEFT_DOWN(ThumbImage::OnLeftDown)
 EVT_LEFT_UP(ThumbImage::OnLeftUp)
 EVT_ENTER_WINDOW(ThumbImage::OnEnter)
