@@ -7,8 +7,8 @@
  * any later version.
  * 
  * @author Lawrence Murray <lawrence@indii.org>
- * $Rev: 518 $
- * $Date: 2013-10-03 20:43:12 +0800 (Thu, 03 Oct 2013) $
+ * $Rev$
+ * $Date$
  */
 #include "ClusterModel.hpp"
 
@@ -90,8 +90,6 @@ void ClusterModel::setNumClusters(const unsigned k) {
     fill(lights.begin(), lights.end(), 0.0f);
     fill(alphas.begin(), alphas.end(), 1.0f);
 
-    delete minClusterer;
-    minClusterer = new KMeansClusterer<>(k, time(NULL));
     cluster();
     notifyAll();
     notifyNumClustersChange();
@@ -268,7 +266,7 @@ void ClusterModel::showAll(const bool on) {
   }
 }
 
-void ClusterModel::calcFg(const wxRect& rect, const int scale, Image& o) {
+void ClusterModel::calcFg(const wxRect& rect, const float scale, Image& o) {
   /* pre-conditions */
   assert(o.getWidth() == rect.width);
   assert(o.getHeight() == rect.height);
@@ -381,10 +379,10 @@ void ClusterModel::calcFg(const wxRect& rect, const int scale, Image& o) {
 }
 
 void ClusterModel::calcAlpha(const unsigned i, const wxRect& rect,
-    const int scale, channel& c) {
+    const float scale, channel& c) {
   /* pre-conditions */
   assert(i < k);
-  assert((int )c.size1() == rect.height && (int )c.size2() == rect.width);
+  assert((int)c.size1() == rect.height && (int)c.size2() == rect.width);
 
   Image* img = res->get(scale);
 
@@ -408,8 +406,7 @@ void ClusterModel::calcAlpha(const unsigned i, const wxRect& rect,
         /* cluster adjustments */
         indii::KMeansClusterer<>::distance_type::prepare(pixel);
         if (minClusterer->assign(pixel) == i) {
-          c(y, x) = uround(
-              255.0f * threshold(hsl(1), saturationDecay, saturationSoftness));
+          c(y, x) = threshold(hsl(1), saturationDecay, saturationSoftness);
         } else {
           c(y, x) = 0;
         }
@@ -419,7 +416,7 @@ void ClusterModel::calcAlpha(const unsigned i, const wxRect& rect,
 }
 
 sparse_mask ClusterModel::calcMask(const unsigned i, const wxRect& rect,
-    const int scale) {
+    const float scale) {
   /* pre-condition */
   assert(i < k);
 
@@ -463,20 +460,11 @@ void ClusterModel::prepare() {
   }
 
   /* prepare data set for clustering */
-  Image* original = res->get();
-  Image* working;
   int originalWidth = res->getWidth();
   int originalHeight = res->getHeight();
   int pixels = originalWidth * originalHeight;
-  int scale = (pixels + maxPixels - 1) / maxPixels;
-
-  if (scale <= 1) {
-    /* image size doesn't exceed limit, cluster it directly */
-    working = new Image(*original, 1);
-  } else {
-    /* image size exceeds limit, cluster scaled version */
-    working = new Image(*original, scale);
-  }
+  float scale = static_cast<float>(pixels) / maxPixels;
+  Image* working = res->getLow(scale);
   int workingWidth = working->getWidth();
   int workingHeight = working->getHeight();
 
@@ -507,7 +495,6 @@ void ClusterModel::prepare() {
       }
     }
   }
-  delete working;
 }
 
 void ClusterModel::cluster() {
@@ -534,8 +521,8 @@ void ClusterModel::cluster() {
 #pragma omp critical
       {
         if (error < minError) {
-          delete minClusterer;
           minError = error;
+          delete minClusterer;
           minClusterer = clusterer;
         } else {
           delete clusterer;
